"""Fix cron fields values, follow-up to NAS-125384

Revision ID: ea024b5dff95
Revises: 6a7c2281f48e
Create Date: 2024-04-03 14:10:41.168534+00:00

"""
from alembic import op
import re
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'ea024b5dff95'
down_revision = '6a7c2281f48e'
branch_labels = None
depends_on = None


def upgrade():
    conn = op.get_bind()
    for table, prefix in [
        ("tasks_cronjob", "cron_"),
        ("storage_replication", "repl_schedule_"),
        ("storage_replication", "repl_restrict_schedule_"),
        ("tasks_rsync", "rsync_"),
        ("storage_task", "task_"),
        ("storage_scrub", "scrub_"),
    ]:
        fields = [prefix + field for field in ["minute", "hour", "daymonth", "month", "dayweek"]]
        for row in map(dict, conn.execute(f"SELECT id, {', '.join(fields)} FROM {table}").fetchall()):
            for k in fields:
                value = row[k]
                if value is not None and '/' in value and not re.match(r'^(\*|[0-9]+-[0-9]+)/([0-9]+)$', value):
                    if m := re.search(r'/([0-9]+)$', value):
                        value = f'*/{m.group(1)}'
                    else:
                        value = '1'  # No luck in guessing the correct value, here is our best guess

                    conn.execute(f"UPDATE {table} SET {k} = ? WHERE id = ?", [value, row["id"]])



def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    pass
    # ### end Alembic commands ###
